###############################################################################
###############################################################################
##                              getWizard.tcl
###############################################################################
###############################################################################
## Includes procedures to implement the configuration wizard, it was created 
## using tkwizard and tkwizthing by Bryan Oakley oakley_at_bardo.clearlight.com
###############################################################################
###############################################################################
## (c) 2001 Andrs Garca Garca. fandom@retemail.es
## Distributed under the terms of the GPL v2
###############################################################################
###############################################################################

source [file join $dirGetleft(scripts) tkwizard.tcl]

tkwizard::tkwizard .wizard -title {Configure Getleft}

.wizard eval {
    variable wizData
    variable logo

    # default values
    if {[info exists wizData]} {
        unset wizData
    }

    set logo [image create photo \
            -file [file join "$dirGetleft(doc)" logo.gif]]

    bind .wizard <<WizCancel>> "
        tkwizard::.wizard::checkCancel
        break
    "
}

.wizard step {Language} -layout basic {
    global   getleftOptions
    variable wizData

    set c [$this widget clientArea]

    set radioEng [radiobutton $c.radioEng -text English     -value en  \
            -variable getleftOptions(lang)                             \
            -command "tkwizard::.wizard::changeLanguage en"]
    set radioChi [radiobutton $c.radioChi -text Chinese(T)  -value zht \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage zht"]
    set radioDeu [radiobutton $c.radioDeu -text Deutsch     -value de  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage de"]
    set radioEsp [radiobutton $c.radioEsp -text Espaol     -value es  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage es"]
    set radioEso [radiobutton $c.radioEso -text Esperanto   -value eo  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage eo"]
    set radioFra [radiobutton $c.radioFra -text Franais    -value fr  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage fr"]
    set radioIta [radiobutton $c.radioIta -text Italiano    -value it  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage it"]
    set radioKor [radiobutton $c.radioKor -text Korean      -value ko  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage ko"]
    set radioNed [radiobutton $c.radioNed -text Nederlands  -value nl  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage nl"]
    set radioPor [radiobutton $c.radioPor -text Portugues   -value pt  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage pt"]
    set radioRus [radiobutton $c.radioRus -text Russian     -value ru  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage ru"]
    set radioTur [radiobutton $c.radioTur -text Trke      -value tr  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage tr"]
    set radioZas [radiobutton $c.radioZas -text Zasoby      -value pl  \
            -variable getleftOptions(lang) \
            -command "tkwizard::.wizard::changeLanguage pl"]

    if {![info exists getleftOptions(lang)]} {
        set getleftOptions(lang) en
    }
    changeLanguage $getleftOptions(lang)

    grid $radioEng $radioChi $radioDeu -sticky w -padx 5
    grid $radioEsp $radioEso $radioFra -sticky w -padx 5
    grid $radioIta $radioKor $radioNed -sticky w -padx 5
    grid $radioPor $radioRus $radioTur -sticky w -padx 5
    grid $radioZas                     -sticky w -padx 5

    return

}
            
.wizard step {Proxy} -layout basic {
    variable wizData
    global   confWizardLabels

    set c [$this widget clientArea]
    $this stepconfigure                                     \
        -title    $confWizardLabels(proxTitle)              \
        -subtitle $confWizardLabels(proxSub)                \
        -pretext  {}                                        \
        -posttext {}

    Herramientas::ConfProxyWindowCommon $c

    bind .wizard <<WizFinish>> "
        [namespace current]::proxyControl $c
        break
    "

    return
}

################################################################################
# proxyControl
#     Inovokes the procedure that checks that the data entered for the proxy
#     is consistent.
#
# Parameter
#     clientArea: the widget in which the dialog is written.
################################################################################
proc tkwizard::.wizard::proxyControl {clientArea} {
    set exitCode [::Herramientas::ConfProxyControl accept $clientArea]
    if {$exitCode==0} {
        destroy .wizard
    }
    return
}

################################################################################
# changeLanguage
#     When the user click on one of the language options, this procedure takes
#     care of actually changing the texts.
#
# Parameter
#     lan: code of the language, 'es', 'en', ...
################################################################################
proc tkwizard::.wizard::changeLanguage {lan} {
    global confWizardLabels getleftOptions
    variable logo

    ReadLangFile $lan

    wm title .wizard $confWizardLabels(title)

    .wizard stepconfigure                                   \
        -title    $confWizardLabels(lanTitle)               \
        -subtitle $confWizardLabels(lanSub)                 \
        -icon     $logo                                     \
        -pretext  {}                                        \
        -posttext {}

    return
}

################################################################################
# checkCancel
#     When the user click on cancel or tries to close the window, we will first
#     check that he really means it.
################################################################################
proc tkwizard::.wizard::checkCancel {} {
    global confWizardLabels

    set what [tk_messageBox -type yesno -icon question \
            -message $confWizardLabels(sureMssg)       \
            -title $confWizardLabels(sureTitle)]
    if {$what!="no"} {
        exit
    }
    focus .wizard
    return
}


.wizard show
tkwait window .wizard 
            
